"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpResponse = exports.HttpClient = void 0;
const ApiClient_1 = require("../errors/ApiClient");
class HttpClient {
    constructor(shop) {
        this.shop = shop;
        this.storage = {
            token: null,
            expiresIn: null
        };
    }
    async get(url, headers = {}) {
        return await this.request('GET', url, null, headers);
    }
    async post(url, json = {}, headers = {}) {
        headers['content-type'] = 'application/json';
        headers['accept'] = 'application/json';
        return await this.request('POST', url, JSON.stringify(json), headers);
    }
    async put(url, json = {}, headers = {}) {
        headers['content-type'] = 'application/json';
        headers['accept'] = 'application/json';
        return await this.request('PUT', url, JSON.stringify(json), headers);
    }
    async patch(url, json = {}, headers = {}) {
        headers['content-type'] = 'application/json';
        headers['accept'] = 'application/json';
        return await this.request('PATCH', url, JSON.stringify(json), headers);
    }
    async delete(url, json = {}, headers = {}) {
        headers['content-type'] = 'application/json';
        headers['accept'] = 'application/json';
        return await this.request('DELETE', url, JSON.stringify(json), headers);
    }
    async request(method, url, body = '', headers = {}) {
        const fHeaders = Object.assign({}, headers);
        fHeaders['Authorization'] = `Bearer ${await this.getToken()}`;
        const f = await globalThis.fetch(`${this.shop.shopUrl}/api${url}`, {
            body,
            headers: fHeaders,
            method
        });
        // Obtain new token
        if (!f.ok && f.status === 401) {
            this.storage.expiresIn = null;
            return await this.request(method, url, body, headers);
        }
        else if (!f.ok) {
            throw new ApiClient_1.ApiClientRequestFailed(this.shop.id, new HttpResponse(f.status, await f.json(), f.headers));
        }
        if (f.status === 204) {
            return new HttpResponse(f.status, {}, f.headers);
        }
        return new HttpResponse(f.status, await f.json(), f.headers);
    }
    async getToken() {
        if (this.storage.expiresIn === null) {
            const auth = await globalThis.fetch(`${this.shop.shopUrl}/api/oauth/token`, {
                method: 'POST',
                headers: {
                    'content-type': 'application/json',
                },
                body: JSON.stringify({
                    grant_type: 'client_credentials',
                    client_id: this.shop.clientId,
                    client_secret: this.shop.clientSecret
                })
            });
            if (!auth.ok) {
                throw new ApiClient_1.ApiClientAuthenticationFailed(this.shop.id, new HttpResponse(auth.status, (await auth.json()), auth.headers));
            }
            const expireDate = new Date();
            const authBody = await auth.json();
            this.storage.token = authBody.access_token;
            expireDate.setSeconds(expireDate.getSeconds() + authBody.expires_in);
            this.storage.expiresIn = expireDate;
            return this.storage.token;
        }
        if (this.storage.expiresIn.getTime() < (new Date()).getTime()) {
            // Expired
            this.storage.expiresIn = null;
            return await this.getToken();
        }
        return this.storage.token;
    }
}
exports.HttpClient = HttpClient;
class HttpResponse {
    constructor(statusCode, body, headers) {
        this.statusCode = statusCode;
        this.body = body;
        this.headers = headers;
    }
}
exports.HttpResponse = HttpResponse;
